VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CCollectionDoc"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************
'* CCollectionDoc.cls                               *
'* provides a document class to store the function  *
'* collection data                                  *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 14:52                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

' data header of the document to store
Private Type CollectionDocHeader
    Author  As String * 50
    Title   As String * 50
    Created As Date
    Changed As Date
End Type

Private Const DEF_TITLE     As String = "Functions"
Private Const DEF_AUTHOR    As String = "Konrad Rudolph"
'----------------------------------------------------

Private m_FileIO    As FileAccess.CFile
Private m_Header    As CollectionDocHeader
Private m_Data      As Collection
'----------------------------------------------------

Private Sub Class_Initialize()
    ' initialise file object
    Set m_FileIO = New CFile
    
    ' initialise header with default values
    m_Header.Author = DEF_AUTHOR
    m_Header.Title = DEF_TITLE
    m_Header.Created = Now
    m_Header.Changed = Now
    
    Set m_Data = New Collection
End Sub

Private Sub Class_Terminate()
    Set m_FileIO = Nothing
    Set m_Data = Nothing
End Sub
'----------------------------------------------------

Public Property Get Title() As String
    Title = Trim(m_Header.Title)
End Property
Public Property Let Title(RHS As String)
    m_Header.Title = RHS
End Property

Public Property Get Author() As String
    Author = Trim$(m_Header.Author)
End Property
Public Property Let Author(RHS As String)
    m_Header.Author = RHS
End Property

Public Property Get Created() As Date
    Created = m_Header.Created
End Property

Public Property Get Changed() As Date
    Changed = m_Header.Changed
End Property
'----------------------------------------------------

'----------------------------------------------------
' SetData
' assigns functions to this document
' - [IN] Data AS ListItems: functions to assign
'----------------------------------------------------
Public Sub SetData(Data As ListItems)
    Dim Item As ListItem
    
    ' erase collection
    Set m_Data = New Collection
    
    If Data Is Nothing Then _
        Exit Sub
    
    For Each Item In Data
        Call m_Data.Add(Item.Text)
    Next
End Sub

'----------------------------------------------------
' GetData As Collection
' - returns a collection of functions in this document
'----------------------------------------------------
Public Function GetData() As Collection
    Dim Item As Variant
    
    Set GetData = m_Data
End Function

'----------------------------------------------------
' Load As Boolean
' loads a document file
' - returns success
'----------------------------------------------------
Public Function Load(FileName As String) As Boolean
    Dim Data()  As String
    Dim I       As Long
    
    If Not CBool( _
        m_FileIO.OpenFile(FileName, OF_OPENEXISTING) _
    ) Then _
        Exit Function   ' error opening file
    
    If Not m_FileIO.InputStream.ReadStruct( _
        VarPtr(m_Header), _
        LenB(m_Header) _
    ) Then _
        Exit Function   ' error reading header
    
    Data = Split(m_FileIO.InputStream.ReadToEnd(), vbNullChar)
    Set m_Data = New Collection
    
    For I = 0 To UBound(Data)
        Call m_Data.Add(Data(I))
    Next I
    
    Load = True
End Function

'----------------------------------------------------
' Save As Boolean
' saves the header + data of the document in a file
' - [IN] FileName as String: path to a file
' - returns success
'----------------------------------------------------
Public Function Save(FileName As String) As Boolean
    Dim Data    As String
    Dim Item    As Variant
    
    m_Header.Changed = Now
    
    If Not CBool(m_FileIO.OpenFile(FileName, OF_CREATENEW)) Then _
        Exit Function   ' unable to create file
    
    ' put header
    Call m_FileIO.OutputStream.WriteStruct( _
        VarPtr(m_Header), _
        LenB(m_Header) _
    )
    
    For Each Item In m_Data
        Data = Data & CStr(Item) & vbNullChar
    Next Item
    If Data <> "" Then
        Data = Left$(Data, Len(Data) - 1)
        
        ' put data
        Call m_FileIO.OutputStream.WriteString(Data)
    End If ' Data <> ""
    
    Save = True
End Function
