VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CLandScape"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************
'* CLandScape.cls                                   *
'* transforms a lanscape array (2D) into a 1D       *
'* vertex trianglestrip for the CSpace class        *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 14:51                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Private m_LandData()    As Double
Private m_Target        As CSpace
Private m_hObject       As Handle
Private m_lWidth        As Long, _
        m_lHeight       As Long
'----------------------------------------------------

Public Property Get LandData(X As Long, Y As Long) As Double
    LandData = m_LandData(X, Y)
End Property
Public Property Let LandData(X As Long, Y As Long, RHS As Double)
    m_LandData(X, Y) = RHS
End Property

Public Property Get Target() As CSpace
    Set Target = m_Target
End Property
Public Property Set Target(RHS As CSpace)
    Set m_Target = RHS
End Property

Public Property Get hObject() As Handle
    hObject = m_hObject
End Property
'----------------------------------------------------

'----------------------------------------------------
' Create
' creates a new landscape
' - [IN] Width As Long: width of the lanscape
' - [IN] Height As Long: height of the landscape
'----------------------------------------------------
Public Sub Create(Width As Long, Height As Long)
    Call Destroy
    
    m_lWidth = Width
    m_lHeight = Height
    
    ReDim m_LandData(m_lWidth - 1, m_lHeight - 1) As Double
End Sub

'----------------------------------------------------
' Destroy
' destroys the old landscape
'----------------------------------------------------
Public Sub Destroy()
    If m_hObject <> NULL_HANDLE Then
        Call m_Target.Remove(m_hObject)
        m_hObject = NULL_HANDLE
    End If
    
    m_lWidth = 0
    m_lHeight = 0
    Erase m_LandData
End Sub

'----------------------------------------------------
' CreateStrip As Handle
' creates a trianglestrip from the landscape data
' (i.e. transform the 2D array into a 1D list of
' 3D vertices that define the grid)
' - returns a handle to the trianglestrip
'----------------------------------------------------
Public Function CreateStrip() As Handle
    Dim tmpStrip()  As POINT3D
    Dim Col         As Long, _
        Row         As Long
    Dim VertIndex   As Long
    
    ReDim tmpStrip((m_lWidth - 1) * (m_lHeight - 2) * 5 - 1) As POINT3D
    
    ' shape of one reactange of the grid:
    '
    ' 0+1-------1+1
    '  |         |
    '  |         |
    '  |         |
    ' 0+0-------1+0
    
    ' shape of the grid:
    '
    ' +-+-+-+-+-+ ...
    ' | | | | | |
    ' +-+-+-+-+-+
    ' | | | | | |
    ' +-+-+-+-+-+
    ' . . .
    
    For Col = 0 To m_lWidth - 3
        For Row = 0 To m_lHeight - 2
            ' corner 0|0
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
            
            ' corner 1|0
            tmpStrip(VertIndex).X = Col + 1
            tmpStrip(VertIndex).Y = m_LandData(Col + 1, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
            
            ' corner 1|1
            tmpStrip(VertIndex).X = Col + 1
            tmpStrip(VertIndex).Y = m_LandData(Col + 1, Row + 1)
            tmpStrip(VertIndex).z = Row + 1
            VertIndex = VertIndex + 1
            
            ' corner 0|1
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row + 1)
            tmpStrip(VertIndex).z = Row + 1
            VertIndex = VertIndex + 1
            
            ' corner 0|0
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
        Next Row
        
        Col = Col + 1
        
        For Row = m_lHeight - 2 To 0 Step -1
            ' corner 0|0
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
            
            ' corner 1|0
            tmpStrip(VertIndex).X = Col + 1
            tmpStrip(VertIndex).Y = m_LandData(Col + 1, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
            
            ' corner 1|1
            tmpStrip(VertIndex).X = Col + 1
            tmpStrip(VertIndex).Y = m_LandData(Col + 1, Row + 1)
            tmpStrip(VertIndex).z = Row + 1
            VertIndex = VertIndex + 1
            
            ' corner 0|1
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row + 1)
            tmpStrip(VertIndex).z = Row + 1
            VertIndex = VertIndex + 1
            
            ' corner 0|0
            tmpStrip(VertIndex).X = Col
            tmpStrip(VertIndex).Y = m_LandData(Col, Row)
            tmpStrip(VertIndex).z = Row
            VertIndex = VertIndex + 1
        Next Row
    Next Col
    
    If m_hObject <> NULL_HANDLE Then _
        Call m_Target.Remove(m_hObject)
    m_hObject = m_Target.Add(tmpStrip)
    
    CreateStrip = m_hObject
End Function
