VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CPlotFunction"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'****************************************************
'* CPlotFunction.cls                                *
'* "core class" of the function plotter             *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 14:52                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Public m_Drawing    As GDI_Draw.CDraw
Public m_Space      As GDI_Draw.CSpace
Public m_hGrid      As Handle
Public m_Parser     As FunctionParser.CParser

Public m_lResX     As Long, _
       m_lResY     As Long      ' stores the rendering resolution
Public m_dMinX     As Double, _
       m_dMaxX     As Double, _
       m_dMinY     As Double, _
       m_dMaxY     As Double    ' stores the edge values of the surface

Private m_Grid      As CLandScape

Private m_sFormula  As String
Private m_bShowForm As Boolean
'----------------------------------------------------

Public Event Error( _
  Source As String, _
  Description As String _
):  ' error event
'----------------------------------------------------

Private Sub Class_Initialize()
    ' assign some default values
    m_lResX = 10
    m_lResY = 10
    m_dMinX = -5
    m_dMaxX = 5
    m_dMinY = -5
    m_dMaxY = 5
End Sub

Private Sub Class_Terminate()
    Set m_Parser = Nothing
    Set m_Space = Nothing
    Set m_Drawing = Nothing
End Sub
'----------------------------------------------------

Public Property Get Formula() As String
    Formula = m_sFormula
End Property
Public Property Let Formula(RHS As String)
    If m_Parser Is Nothing Then _
        Exit Property
    
    m_sFormula = RHS
    
    Call AssignFormula
End Property

Public Property Get ResX() As Long
    ResX = m_lResX
End Property
Public Property Let ResX(RHS As Long)
    m_lResX = RHS
    
    If m_lResX Mod 2 = 1 Then _
        m_lResX = m_lResX - 1
    
    If m_lResX < 10 Then _
        m_lResX = 10
    If m_lResX > 40 Then _
        m_lResX = 40
    
    If Not m_Grid Is Nothing Then
        Call m_Grid.Destroy
        Call m_Grid.Create(m_lResX, m_lResY)
    End If
End Property

Public Property Get ResY() As Long
    ResY = m_lResY
End Property
Public Property Let ResY(RHS As Long)
    m_lResY = RHS
    
    If m_lResY Mod 2 = 1 Then _
        m_lResY = m_lResY - 1
    
    If m_lResY < 10 Then _
        m_lResY = 10
    If m_lResY > 40 Then _
        m_lResY = 40
    
    If Not m_Grid Is Nothing Then
        Call m_Grid.Destroy
        Call m_Grid.Create(m_lResX, m_lResY)
    End If
End Property

Public Property Get hGrid() As Handle
    hGrid = m_hGrid
End Property
'----------------------------------------------------

'----------------------------------------------------
' Initialise
' initialises the class
' - [IN] Target As PictureBox: target picturebox
'----------------------------------------------------
Public Sub Initialise(Target As PictureBox)
    ' initialise CDraw class
    Set m_Drawing = NewCDraw(Target)
    
    ' initalise space
    Set m_Space = NewCSpace(m_Drawing, PlotLineColour)
    m_Space.ViewPort = MakeRect(-50, 50, 50, -50)
    
    ' initalise landscape
    Set m_Grid = New CLandScape
    Set m_Grid.Target = m_Space
    Call m_Grid.Create(m_lResX, m_lResY)
    
    ' initialise function parser
    Set m_Parser = New CParser
    Call m_Parser.Variables.Add("x")
    Call m_Parser.Variables.Add("y")
End Sub

'----------------------------------------------------
' SetRenderLimits
' assign rendering limits
' ...
'----------------------------------------------------
Public Sub SetRenderLimits( _
  MinX As Double, _
  MaxX As Double, _
  MinY As Double, _
  MaxY As Double _
)
    m_dMinX = MinX
    m_dMaxX = MaxX
    m_dMinY = MinY
    m_dMaxY = MaxY
    
    Call AssignFormula
End Sub

'----------------------------------------------------
' Render
' do the rendering
'----------------------------------------------------
Public Sub Render()
    ' clear the backbuffer
    Call m_Drawing.Clear(PlotBackColour)
    
    ' render the wireframe
    If m_bShowForm Then _
        Call m_Space.Render
    
    ' flip backbuffer to front buffer
    Call m_Drawing.Flip
End Sub
'----------------------------------------------------

'----------------------------------------------------
' AssignFormula
' a new function has been assigned - create grid!
'----------------------------------------------------
Private Sub AssignFormula()
    Dim DiffX   As Long, _
        DiffY   As Long
    Dim ret     As ParseErrors
    
    If m_sFormula = "" Then
        ' clear everything
        Call m_Parser.Reset
        Call m_Space.Remove(m_hGrid)
        
        Exit Sub
    End If ' m_sFormula = ""
    
    ' parse the function
    If UsesNotation = UN_Infix Then _
        ret = m_Parser.ParseInfix(m_sFormula) _
    Else _
        ret = m_Parser.ParsePostfix(m_sFormula)
    
    If ret <> PE_NoError Then _
        RaiseEvent Error( _
            "CPlotFunction::Parser::Parse", _
            "Error in function: " & ParseErrorToString(ret) _
        )
    
    Call FillGrid
    
    ' create the grid
    m_hGrid = m_Grid.CreateStrip()
    
    DiffX = m_dMaxX - m_dMinX
    DiffY = m_dMaxY - m_dMinY
    
    ' translate to origin
    Call m_Space.Translate( _
        m_hGrid, _
        MakeVertex( _
            -DiffX / 2 * m_lResX / DiffX, , _
            -DiffY / 2 * m_lResY / DiffY _
        ) _
    )
    
    Call InitCoordinateSystem(World:=m_Space)
    m_bShowForm = True
End Sub

'----------------------------------------------------
' FillGrid
' fills the grid with the z-values corresponding
' to each x and y value by the function
'----------------------------------------------------
Private Sub FillGrid()
    Dim X       As Long, _
        Y       As Long
    Dim StepX   As Double, _
        StepY   As Double
    
    On Error GoTo ERR_FillGrid
    
    StepX = (m_dMaxX - m_dMinX) / (m_lResX - 1)
    StepY = (m_dMaxY - m_dMinY) / (m_lResY - 1)
    
    ' fill in values
    For X = 0 To m_lResX - 1
        For Y = 0 To m_lResY - 1
            m_Parser.Variables.Value("x") = X * StepX + m_dMinX
            m_Parser.Variables.Value("y") = Y * StepY + m_dMinY
            m_Grid.LandData(X, Y) = m_Parser.Execute
        Next Y
    Next X
        
    On Error GoTo 0
    Exit Sub

ERR_FillGrid:
    RaiseEvent Error( _
        "CPlotFunction::Parser::Execute", _
        "Error in function: " & _
        ParseErrorToString(m_Parser.LastError) _
    )
End Sub
