Attribute VB_Name = "modGlobals"
Option Explicit

' comctl32.dll
Public Declare Sub InitCommonControls Lib "comctl32.dll" ()

' gdi32.dll
Public Declare Function StretchBlt Lib "gdi32.dll" ( _
    ByVal hDC As hDC, _
    ByVal X As Long, _
    ByVal Y As Long, _
    ByVal nWidth As Long, _
    ByVal nHeight As Long, _
    ByVal hSrcDC As hDC, _
    ByVal xSrc As Long, _
    ByVal ySrc As Long, _
    ByVal nSrcWidth As Long, _
    ByVal nSrcHeight As Long, _
    ByVal dwRop As RasterOpConstants _
) As BOOL
Public Declare Function CreateCompatibleBitmap Lib "gdi32.dll" ( _
    ByVal hDC As hDC, _
    ByVal nWidth As Long, _
    ByVal nHeight As Long _
) As Handle
Public Declare Function SelectObject Lib "gdi32.dll" ( _
    ByVal hDC As hDC, _
    ByVal hObject As Long _
) As Handle
'----------------------------------------------------

Public Enum UseNotation
    UN_Infix
    UN_Postfix
End Enum
'----------------------------------------------------

Public XMLPropBag       As FileAccess.CTree
Public CollDoc          As CCollectionDoc
Public hCoordSys(2)     As Handle

' some general settings
Public PlotLineColour   As OLE_COLOR
Public PlotBackColour   As OLE_COLOR

Public UsesNotation     As UseNotation

Public LastOpen         As String
'----------------------------------------------------

Public Sub Main()
    Dim argv()  As String
    
    ' initialise settings property bag
    Set XMLPropBag = New CTree
    
    ' initialise document class
    Set CollDoc = New CCollectionDoc
    
    ' read the program settings
    Call ReadSettings
    
    If Command$ <> "" Then
        argv = SplitCommandLine()
        
        ' quick'n'dirty test if command contains file
        ' if so, load it
        If Dir$(argv(0)) <> "" Then _
            LastOpen = argv(0)
    End If
End Sub
'----------------------------------------------------

'----------------------------------------------------
' ReadSettings
' loads the program settings from the property bag
'----------------------------------------------------
Public Sub ReadSettings()
    Dim sUsesNotation   As String
    
    With XMLPropBag
        If Not .ImportXML(App.Path & "\settings.conf") Then
            ' path does not exist or file is ill formatted
            Call AssignDefaults
            Exit Sub
        End If
        
        LastOpen = .Item("Settings\LastOpen").Value
        
        PlotLineColour = .Item("Settings\Plot\LineColour").Value
        PlotBackColour = .Item("Settings\Plot\BackColour").Value
        sUsesNotation = .Item("Settings\Parser\Notation").Value
        
        If UCase$(sUsesNotation) = "INFIX" Then _
            UsesNotation = UN_Infix _
        Else _
            UsesNotation = UN_Postfix
    End With ' XMLPropBag
    
    dlgMain.picPlot.BackColor = PlotBackColour
End Sub

'----------------------------------------------------
' AssignDefaults
' assigns default setting values
'----------------------------------------------------
Public Sub AssignDefaults()
    PlotLineColour = vbWhite
    PlotBackColour = vbBlack
    
    dlgMain.picPlot.BackColor = PlotBackColour
    
    UsesNotation = UN_Infix
End Sub

'----------------------------------------------------
' WriteSettings
' stores the settings in the property bag
'----------------------------------------------------
Public Sub WriteSettings()
    Dim sUsesNotation   As String
    
    With XMLPropBag
        Call .Clear
        
        Call .Add("Settings")
        Call .Add("Settings\LastOpen", LastOpen)
        Call .Add("Settings\Plot")
        Call .Add("Settings\Plot\LineColour", CStr(PlotLineColour))
        Call .Add("Settings\Plot\BackColour", CStr(PlotBackColour))
        
        If UsesNotation = UN_Infix Then _
            sUsesNotation = "Infix" _
        Else _
            sUsesNotation = "Postfix"
        Call .Add("Settings\Parser")
        Call .Add("Settings\Parser\Notation", sUsesNotation)
        
        If Not .ExportXML(App.Path & "\settings.conf") Then
            Call MsgBox( _
                "Sorry: Unable to write settings.", _
                vbInformation, _
                "Error" _
            )
        End If
    End With
End Sub

'----------------------------------------------------
' InitCoordinateSystem
' creates a simple coordinate system
' consisting of three lines, one for each axis
' - [IN] World As CSpace: instance of CSpace to draw into
'----------------------------------------------------
Public Sub InitCoordinateSystem(World As GDI_Draw.CSpace)
    Static bExist   As Boolean
    Dim Line3D(1)   As POINT3D
    
    If bExist Then
        ' clear up
        Call World.Remove(hCoordSys(0))
        Call World.Remove(hCoordSys(1))
        Call World.Remove(hCoordSys(2))
    End If
    
    ' x axis
    Line3D(0) = MakeVertex(-1000)
    Line3D(1) = MakeVertex(1000)
    hCoordSys(0) = World.Add(Line3D)
    
    ' y axis
    Line3D(0) = MakeVertex(, -1000)
    Line3D(1) = MakeVertex(, 1000)
    hCoordSys(1) = World.Add(Line3D)
    
    ' z axis
    Line3D(0) = MakeVertex(, , -1000)
    Line3D(1) = MakeVertex(, , 1000)
    hCoordSys(2) = World.Add(Line3D)
    
    bExist = True
End Sub

'----------------------------------------------------
' ParseErrorToString As String
' converts a parse error into a string expression
' - [IN]: Error As ParseError: parse error code
' - returns the string expression
'----------------------------------------------------
Public Function ParseErrorToString(Error As ParseErrors) As String
    Select Case Error
        Case PE_MissingLeftBracket
            ParseErrorToString = _
                "Left bracket is missing."
        Case PE_MissingRightBracket
            ParseErrorToString = _
                "Right bracket is missing."
        Case PE_InvalidTerm
            ParseErrorToString = _
                "Function contains invalid term."
        Case PE_StackError
            ParseErrorToString = _
                "Function contains stack error."
    End Select
End Function

'----------------------------------------------------
' SplitCommandLine As String()
' splits the commandline arguments into an array
' - [IN] [Value As String]: string to split;
'   if missing take Command$ instead
' - returns an array of split values
'----------------------------------------------------
Public Function SplitCommandLine( _
  Optional Value As String _
) As String()
    Dim I           As Long
    Dim Char        As String * 1
    Dim bQuote      As Boolean
    Dim Result()    As String
    Dim CurrVal     As Long
    
    CurrVal = -1
    
    If LenB(Value) = 0 Then _
        Value = Command$
    
    For I = 1 To LenB(Value) Step 2
        Char = MidB$(Value, I, 2)
        
        Select Case Char
        Case " ", vbTab
            If bQuote Then
                GoSub AddVal
            Else
                If CurrVal + 1 Then
                    If LenB(Result(CurrVal)) Then
                        CurrVal = CurrVal + 1
                        ReDim Preserve Result(CurrVal) As String
                    End If ' LenB()
                End If ' CurrVal + 1
            End If ' bQuot
        Case """"
            bQuote = Not bQuote
        Case Else
            GoSub AddVal
        End Select ' Char
    Next I
    
    SplitCommandLine = Result
    
    Exit Function
    '------------------------------------------------

AddVal:
    If CurrVal = -1 Then
        CurrVal = 0
        ReDim Result(0) As String
    End If
    
    Result(CurrVal) = Result(CurrVal) & Char
Return ' AddVal
End Function

'----------------------------------------------------
' TranslateGraph
' translates the plot
'----------------------------------------------------
Public Sub TranslateGraph( _
  World As GDI_Draw.CSpace, _
  Grid As Handle, _
  Vector As POINT3D _
)
    Call World.Translate(Grid, Vector)
    Call World.Translate(hCoordSys(0), Vector)
    Call World.Translate(hCoordSys(1), Vector)
    Call World.Translate(hCoordSys(2), Vector)
End Sub

'----------------------------------------------------
' StretchGraph
' stretches or shrinks the plot
'----------------------------------------------------
Public Sub StretchGraph( _
  World As GDI_Draw.CSpace, _
  Grid As Handle, _
  Factor As Double _
)
    Call World.Stretch(Grid, Factor)
    Call World.Stretch(hCoordSys(0), Factor)
    Call World.Stretch(hCoordSys(1), Factor)
    Call World.Stretch(hCoordSys(2), Factor)
End Sub

'----------------------------------------------------
' RotateGraph
' rotates the plot
'----------------------------------------------------
Public Sub RotateGraph( _
  World As GDI_Draw.CSpace, _
  Grid As Handle, _
  Alpha As Long, _
  Beta As Long, _
  Gamma As Long _
)
    Call World.Rotate(Grid, Alpha, Beta, Gamma)
    Call World.Rotate(hCoordSys(0), Alpha, Beta, Gamma)
    Call World.Rotate(hCoordSys(1), Alpha, Beta, Gamma)
    Call World.Rotate(hCoordSys(2), Alpha, Beta, Gamma)
End Sub
