VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CWriteStream"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private Declare Sub Peek Lib "kernel32" Alias "RtlMoveMemory" ( _
  nDest As Any, _
  ByVal lpSrc As Long, _
  ByVal nLen As Long _
)

Public Enum OverrideMode
    OM_OVERRIDE             ' writes at pointer, overwrites existing data
    OM_APPEND               ' writes at EOF position
    OM_INSERT               ' writes at pointer, shifts existing data
End Enum
#If False Then
    Const OM_OVERRIDEMODE = 0
    Const OM_APPEND = 1
    Const OM_INSERT = 2
#End If

Private mHandle     As Long
Private mPos        As Long

'----------------------------------------------------
Friend Sub init__(Handle As Long)
    mHandle = Handle
    mPos = 1
End Sub

'----------------------------------------------------

'----------------------------------------------------
' WriteLine [void]
' writes a line (string + carriage return) to the file
' Line As String: Text to write
' Unix As Boolean: if True, the function takes vbCr
'   instead of vbCrLf
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteLine( _
  Line As String, _
  Optional Unix As Boolean = False, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    Dim LineLine As String
    
    If mHandle Then
        If Unix Then _
            LineLine = Line & vbLf _
        Else _
            LineLine = Line & vbCrLf
        
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, LineLine
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, LineLine
            mPos = mPos + LenB(LineLine) \ 2
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, LineLine
            mPos = mPos + LenB(LineLine) \ 2
        End Select
    End If
End Sub

'----------------------------------------------------
' WriteString [void]
' writes a text to the file
' Text As String: Text to write
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteString( _
  Text As String, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    If mHandle Then
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, Text
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, Text
            mPos = mPos + LenB(Text) \ 2
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, Text
            mPos = mPos + LenB(Text) \ 2
        End Select
    End If
End Sub

'----------------------------------------------------
' WriteBuffer [void]
' writes a byte buffer to the file
' Buffer() As Byte: buffer to write
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteBuffer( _
  Buffer() As Byte, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    If mHandle Then
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, Buffer
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, Buffer
            mPos = mPos + UBound(Buffer) - LBound(Buffer) + 1
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, Buffer
            mPos = mPos + UBound(Buffer) - LBound(Buffer) + 1
        End Select
    End If
End Sub

'----------------------------------------------------
' WriteStruct [void]
' writes a structure (type) to the file
' lpStruct As Long: long pointer to the structure to write
' Size: size of the struct in bytes
'   !!! size is LenB(struct) !!!
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteStruct( _
  lpStruct As Long, _
  Size As Long, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    Dim Buffer() As Byte
    
    If mHandle And Size > 0 Then
        ReDim Buffer(Size - 1) As Byte
        
        Call Peek(Buffer(0), lpStruct, Size)
        
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, Buffer
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, Buffer
            mPos = mPos + Size
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, Buffer
            mPos = mPos + Size
        End Select
    End If
End Sub

'----------------------------------------------------
'----------------------------------------------------
' WriteChar [void]
' writes a byte to the file
' Char As Byte: byte to write
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteChar( _
  Char As Byte, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    If mHandle Then
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, Char
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, Char
            mPos = mPos + 1
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, Char
            mPos = mPos + 1
        End Select
    End If
End Sub

'----------------------------------------------------
' WriteWord [void]
' writes an integer to the file
' Word As Integer: integer to write
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteWord( _
  Word As Integer, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    If mHandle Then
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, Word
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, Word
            mPos = mPos + 2
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, Word
            mPos = mPos + 2
        End Select
    End If
End Sub

'----------------------------------------------------
' WriteDWord [void]
' writes a long to the file
' DWord As Long: long to write
' Mode As OverrideMode: writing mode
'----------------------------------------------------
Public Sub WriteDWord( _
  DWord As Long, _
  Optional Mode As OverrideMode = OM_OVERRIDE _
)
    If mHandle Then
        Select Case Mode
        Case OverrideMode.OM_APPEND
            Put #mHandle, LOF(mHandle) + 1, DWord
        Case OverrideMode.OM_INSERT
            Dim tmpBuff() As Byte
            
            ReDim tmpBuff(LOF(mHandle) - mPos) As Byte
            
            Get #mHandle, mPos, tmpBuff
            Put #mHandle, mPos, DWord
            mPos = mPos + 1
            Put #mHandle, mPos, tmpBuff
        Case OverrideMode.OM_OVERRIDE
            Put #mHandle, mPos, DWord
            mPos = mPos + 1
        End Select
    End If
End Sub

'----------------------------------------------------
'----------------------------------------------------
' Pointer - Property Long
' sets or retrieves the current file write pointer
'   (zerobased)
'----------------------------------------------------
Public Property Get Pointer() As Long
    Pointer = mPos - 1
End Property

Public Property Let Pointer(newPointer As Long)
    If mHandle Then
        mPos = newPointer + 1
        
        If mPos < 1 Then _
            mPos = 1
        If mPos > LOF(mHandle) Then _
            mPos = LOF(mHandle)
    End If
End Property

'----------------------------------------------------
' EOF - ReadOnly Property Boolean
' returns whether the end of file is reached
'----------------------------------------------------
Public Property Get EOF() As Boolean
    EOF = (mPos > LOF(mHandle))
End Property

