VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************
'* CFormParser.cls                                  *
'* parses formulas in infix and postfix notation    *
'* supports variables, uses RPN parse tree          *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 13.08.2003 21:56                    *
'* Version: 3.0.1                                   *
'****************************************************
Option Explicit

Private m_First     As CFormItem
Private m_Current   As CFormItem
Private m_Ops(3)    As String
Private m_AllOps    As String
Private m_LastError As ParseErrors
Private m_Vars      As CVariables
'----------------------------------------------------

Private Sub Class_Initialize()
    ' store operators according to their priority
    m_Ops(0) = OP_ADD & OP_DIFF
    m_Ops(1) = OP_MUL & OP_DIV & OP_IDIV & OP_MOD
    m_Ops(2) = OP_POW & OP_ROOT
    m_Ops(3) = OP_AND & OP_OR & OP_XOR
    
    m_AllOps = m_Ops(0) & m_Ops(1) & m_Ops(2) & m_Ops(3)
    
    Set m_Vars = New CVariables
End Sub

Private Sub Class_Terminate()
    Set m_Vars = Nothing
End Sub
'----------------------------------------------------

Public Property Get LastError() As ParseErrors
    LastError = m_LastError
End Property

Public Property Get Variables() As CVariables
    Set Variables = m_Vars
End Property
'----------------------------------------------------

'----------------------------------------------------
' ParseInfix As ParseErrors
' parses a formula in infix (= normal) notation
' - [IN] Formula As String: functioon to parse
' - returns an error code
'----------------------------------------------------
Public Function ParseInfix(Formula As String) As ParseErrors
    Dim tmpForm     As String
    
    m_LastError = PE_NoError
    
    tmpForm = Formula
    Call ConvertSigns_Infix(tmpForm)
    
    Set m_First = New CFormItem
    Set m_Current = m_First
    
    Call BuildStack_Infix(RipFormula(tmpForm))
    
    ParseInfix = m_LastError
End Function

'----------------------------------------------------
' ParsePostfix As ParseErrors
' parses a formula in postfix (= RPN) notation
' - [IN] Formula As String: functioon to parse
' - [IN] [Delimiter As String]: an element delimiter
' - returns an error code
'----------------------------------------------------
Public Function ParsePostfix( _
  Formula As String, _
  Optional Delimiter As String = " " _
) As ParseErrors
    Dim tmpForm As String
    
    m_LastError = PE_NoError
    
    tmpForm = Formula
    Call ConvertSigns_Postfix(tmpForm, Delimiter)
    
    Set m_First = BuildStack_Postfix(Trim$(tmpForm), Delimiter)
    
    ParsePostfix = m_LastError
End Function

'----------------------------------------------------
' Execute As Double
' executes the previously parser function
' - returns the result
'----------------------------------------------------
Public Function Execute() As Double
    If m_LastError <> PE_NoError Then _
        Exit Function
    
    Execute = CollapseStack(m_First)
End Function

'----------------------------------------------------
' Reset
' resets the stack
'----------------------------------------------------
Public Sub Reset()
    Set m_First = Nothing
    m_LastError = PE_NoError
End Sub
'----------------------------------------------------

'----------------------------------------------------
' ConvertSigns_Infix
' converts some possible notation errors
' - [IN/OUT] Form As String: function
'----------------------------------------------------
Private Sub ConvertSigns_Infix(ByRef Form As String)
    Form = Replace(Form, "&&", OP_AND)
    Form = Replace(Form, "||", OP_OR)
    Form = Replace(Form, "^^", OP_XOR)
    Form = Replace(Form, " ", "")
End Sub

'----------------------------------------------------
' ConvertSigns_Postfix
' converts some possible notation errors
' - [IN/OUT] Form As String: function
' - [IN] [Delimier As String]: term delimier
'----------------------------------------------------
Private Sub ConvertSigns_Postfix(ByRef Form As String, Delimiter As String)
    Form = Replace(Form, "&&", OP_AND)
    Form = Replace(Form, "||", OP_OR)
    Form = Replace(Form, "^^", OP_XOR)
    Form = Replace(Form, Delimiter & Delimiter, Delimiter)
End Sub

'----------------------------------------------------
' RipFormula As CFormItem
' rips an infix formula, i.e. splits the elements and
' puts them one by one on a parse tree
' - [IN] Expression As String: function to rip
' - returns the first item of the parse tree
'----------------------------------------------------
Private Function RipFormula(Expression As String) As CFormItem
    Dim Current As CFormItem
    Dim I       As Long, _
        J       As Long
    Dim Char    As String * 1
    Dim Number  As String
    Dim Var     As String
    Dim IsNSign As Boolean      ' set if the last character was an operator token
    
    Set RipFormula = New CFormItem
    Set Current = RipFormula
    
    For I = 1 To LenB(Expression) Step 2
        Char = MidB$(Expression, I, 2)
        
        Select Case True
            Case IsNumeric(Char), Char = "."
                Number = Number & Char
                
                If Not CBool(InStrB( _
                    "0123456789.", MidB$(Expression, I + 2, 2)) _
                ) Or Not CBool(LenB(MidB$(Expression, I + 2, 2))) Then
                    Current.ItemType = IT_Number
                    Current.Value = Val(Number)
                    Set Current.Next_ = New CFormItem
                    Set Current = Current.Next_
                    Number = ""
                End If ' CBool(...)
                
                IsNSign = True
            
            Case (Char = "+" Or Char = "-") And Not IsNSign
                Number = Char
            
            Case CBool(InStrB(m_AllOps, Char))
                Current.ItemType = IT_Operator
                Current.Op = Char
                Set Current.Next_ = New CFormItem
                Set Current = Current.Next_
                
                IsNSign = False
            
            Case Char = "("
                Current.ItemType = IT_OpenBracket
                Set Current.Next_ = New CFormItem
                Set Current = Current.Next_
                
                IsNSign = False
            
            Case Char = ")"
                Current.ItemType = IT_CloseBracket
                Set Current.Next_ = New CFormItem
                Set Current = Current.Next_
                
                IsNSign = True
            
            Case Else
                Number = Number & Char
                
                If CBool(InStrB( _
                    m_AllOps & "(" & ")", MidB$(Expression, I + 2, 2) _
                )) Then
                    If (LeftB$(Number, 2) = "+" Or _
                        LeftB$(Number, 2) = "-") And _
                        LenB(Number) = 4 _
                    Then
                        ' assume variable/const with sign
                        Call ResolveVariable( _
                            RightB$(Number, 2), _
                            Current _
                        )
                        
                        If LeftB$(Number, 2) = "-" Then
                            Current.Value = -Current.Value
                            Current.Op = "-" & Current.Op
                        End If
                    ElseIf LenB(Number) > 2 And _
                        MidB$(Expression, I + 2, 2) = "(" _
                    Then
                        ' assume function
                        Current.ItemType = IT_Function
                        Current.Op = UCase$(Number)
                    ElseIf LenB(Number) = 2 Then
                        Call ResolveVariable(Number, Current)
                    Else
                        m_LastError = PE_InvalidTerm
                        Exit Function
                    End If ' ...
                    
                    Set Current.Next_ = New CFormItem
                    Set Current = Current.Next_
                    
                    Number = ""
                End If ' CBool(...)
                
                IsNSign = True
            
        End Select ' True
    Next I
End Function

'----------------------------------------------------
' BuildStack_Infix
' rearranges the parse tree so it becomes a valid
' postfix command stack instead
' - [IN] Token As CFormItem: first token of the
'   parse tree
'----------------------------------------------------
Private Function BuildStack_Infix( _
  ByRef Token As CFormItem _
)
    Dim Operators   As CStringStack
    Dim Priority    As Long, _
        LastPrior   As Long
    Dim LastFunct   As String
    
    LastPrior = -1
    Set Operators = New CStringStack
    
    ' rearrangement schema:
    ' - always push numbers
    ' - do NOT push operators but put them on a separate stack
    ' - if an operator's priority is inferior to the one of the
    ' last operator on the operator stack then push all
    ' operators on the operator stack onto the parse stack
    ' and put the current operator on the operator stack
    ' - at brackets call this procedure recursively
    ' - finally push all remaining commands
    
    Do ' Until Token.Next_ Is Nothing
        Select Case Token.ItemType
            Case IT_Number
                m_Current.ItemType = IT_Number
                m_Current.Value = Token.Value
                Set m_Current.Next_ = New CFormItem
                Set m_Current = m_Current.Next_
            
            Case IT_Variable
                m_Current.ItemType = IT_Variable
                m_Current.Op = Token.Op
                Set m_Current.Next_ = New CFormItem
                Set m_Current = m_Current.Next_
            
            Case IT_Function
                LastFunct = Token.Op
            
            Case IT_Operator
                Priority = PriorityOf(Token.Op)
                
                If Priority < LastPrior Then
                    Do While CBool(Operators.Count)
                        m_Current.ItemType = IT_Operator
                        m_Current.Op = Operators.Pop
                        Set m_Current.Next_ = New CFormItem
                        Set m_Current = m_Current.Next_
                    Loop ' While CBool(Operators.Count)
                ElseIf Priority = LastPrior Then
                    m_Current.ItemType = IT_Operator
                    m_Current.Op = Operators.Pop
                    Set m_Current.Next_ = New CFormItem
                    Set m_Current = m_Current.Next_
                End If ' Priority <= LastPrior
                
                Call Operators.Push(Token.Op)
                LastPrior = Priority
            
            Case IT_OpenBracket
                Set Token = Token.Next_
                Call BuildStack_Infix(Token)
                
                If CBool(LenB(LastFunct)) Then
                    m_Current.ItemType = IT_Function
                    m_Current.Op = LastFunct
                    Set m_Current.Next_ = New CFormItem
                    Set m_Current = m_Current.Next_
                    
                    LastFunct = ""
                End If ' CBool(LenB(LastFunct))
            
            Case IT_CloseBracket
                Exit Do
            
        End Select ' Token.ItemType
        
        If Not Token.Next_ Is Nothing Then _
            Set Token = Token.Next_
    Loop Until Token.Next_ Is Nothing
    
    Do While CBool(Operators.Count)
        m_Current.ItemType = IT_Operator
        m_Current.Op = Operators.Pop
        Set m_Current.Next_ = New CFormItem
        Set m_Current = m_Current.Next_
    Loop
End Function

'----------------------------------------------------
' PriorityOf As Long
' - [IN] Operator As String: an operator
' - returns the priority of the operator
'----------------------------------------------------
Private Function PriorityOf(Operator As String) As Long
    For PriorityOf = 0 To UBound(m_Ops)
        If CBool(InStrB(m_Ops(PriorityOf), Operator)) Then _
            Exit Function
    Next I
End Function

'----------------------------------------------------
' BuildStack_Postfix As CFormItem
' splits the function string and pushes each token
' on the parse stack
' - [IN] Formula As String: function to parse
' - [IN] Delimiter As String: token delimiter
' - returns the first item of the resulting stack
'----------------------------------------------------
Private Function BuildStack_Postfix( _
  Formula As String, _
  Delimiter As String _
) As CFormItem
    Dim tmpStack()  As String
    Dim CurrIndex   As Long
    Dim CurrItem    As CFormItem
    
    tmpStack = Split(Formula, Delimiter)
    
    Set BuildStack_Postfix = New CFormItem
    Set CurrItem = BuildStack_Postfix
    
    For CurrIndex = 0 To UBound(tmpStack)
        Select Case True
            Case IsNumeric(tmpStack(CurrIndex))
                CurrItem.ItemType = IT_Number
                CurrItem.Value = Val(tmpStack(CurrIndex))
            
            Case CBool(InStr(m_AllOps, tmpStack(CurrIndex)))
                CurrItem.ItemType = IT_Operator
                CurrItem.Op = tmpStack(CurrIndex)
            
            Case Else
                If (LeftB$(tmpStack(CurrIndex), 2) = "+" Or _
                    LeftB$(tmpStack(CurrIndex), 2) = "-") And _
                    LenB(tmpStack(CurrIndex)) = 4 _
                Then
                    ' assume variable/const with sign
                    Call ResolveVariable( _
                        RightB$(tmpStack(CurrIndex), 2), _
                        CurrItem _
                    )
                    
                    If LeftB$(tmpStack(CurrIndex), 2) = "-" Then
                        CurrItem.Value = -CurrItem.Value
                        CurrItem.Op = "-" & CurrItem.Op
                    End If
                ElseIf LenB(tmpStack(CurrIndex)) > 2 Then
                    ' assume function
                    CurrItem.ItemType = IT_Function
                    CurrItem.Op = UCase$(tmpStack(CurrIndex))
                ElseIf LenB(tmpStack(CurrIndex)) = 2 Then
                    Call ResolveVariable(tmpStack(CurrIndex), CurrItem)
                Else
                    m_LastError = PE_InvalidTerm
                    Exit Function
                End If ' ...
            
        End Select ' True
        
        Set CurrItem.Next_ = New CFormItem
        Set CurrItem = CurrItem.Next_
    Next CurrIndex
End Function

'----------------------------------------------------
' CollapseStack As Double
' executes the RPN command stack
' - [IN] StartItem As CFormItem: first item of stack
' - returns the result
'----------------------------------------------------
Private Function CollapseStack(StartItem As CFormItem) As Double
    Dim CurrItem    As CFormItem    ' current command item
    Dim ParseStack  As CDoubleStack ' numeric result stack
    
    On Error GoTo ERR_CollapseStack
    
    Set ParseStack = New CDoubleStack
    Set CurrItem = StartItem
    
    Do Until CurrItem.Next_ Is Nothing
        Select Case CurrItem.ItemType
            Case IT_Number
                Call ParseStack.Push(CurrItem.Value)
            Case IT_Variable
                If LeftB$(CurrItem.Op, 2) = "-" Then _
                    Call ParseStack.Push( _
                        -m_Vars.Value(RightB$(CurrItem.Op, 2)) _
                    ) _
                Else _
                    Call ParseStack.Push(m_Vars.Value(CurrItem.Op))
            Case IT_Operator
                Call ParseStack.Execute(CurrItem.Op)
            Case IT_Function
                Call ParseStack.ExecFunct(CurrItem.Op)
        End Select
        
        Set CurrItem = CurrItem.Next_
    Loop
    
    CollapseStack = ParseStack.Pop
    
    On Error GoTo 0
    Exit Function

ERR_CollapseStack:
    On Error GoTo 0
    m_LastError = PE_StackError
End Function

'----------------------------------------------------
' ResolveVariable
' is called on encounter of a variable or constant
' in the function - replaces constants by their value
' and pushes variables back
' - [IN] VarName As String: variable/constant name
' - [IN/OUT] CurrItem As CFormItem: current stack item
'----------------------------------------------------
Private Sub ResolveVariable( _
  VarName As String, _
  ByRef CurrItem As CFormItem _
)
    Select Case UCase$(VarName)
        Case CS_PI  ' pi
            CurrItem.ItemType = IT_Number
            CurrItem.Value = VAL_PI
        
        Case CS_E   ' Euler
            CurrItem.ItemType = IT_Number
            CurrItem.Value = VAL_E
        
        Case Else   ' variable
            CurrItem.ItemType = IT_Variable
            CurrItem.Op = VarName
        
    End Select ' Ucase$(VarName)
End Sub
