VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CBitMap"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************
'* CBitMap.cls                                      *
'* handles the loading and displaying of bitmaps    *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 15:46                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

' msimg32
Private Declare Function TransparentBlt Lib "msimg32" ( _
  ByVal hDCDest As hDC, _
  ByVal nXOriginDest As Long, _
  ByVal nYOriginDest As Long, _
  ByVal nWidthDest As Long, _
  ByVal nHeightDest As Long, _
  ByVal hDCSrc As hDC, _
  ByVal nXOriginSrc As Long, _
  ByVal nYOriginSrc As Long, _
  ByVal nWidthSrc As Long, _
  ByVal nHeightSrc As Long, _
  ByVal crTransparent As Long _
) As Long
Private Declare Function AlphaBlend Lib "msimg32" ( _
  ByVal hDestDC As hDC, _
  ByVal x As Long, _
  ByVal y As Long, _
  ByVal nWidth As Long, _
  ByVal nHeight As Long, _
  ByVal hSrcDC As hDC, _
  ByVal xSrc As Long, _
  ByVal ySrc As Long, _
  ByVal widthSrc As Long, _
  ByVal heightSrc As Long, _
  ByVal blendFunct As Long _
) As BOOL

' gdi32
Private Declare Function CreateBitmap Lib "gdi32" ( _
  ByVal nWidth As Long, _
  ByVal nHeight As Long, _
  ByVal nPlanes As Long, _
  ByVal nBitCount As Long, _
  lpBits As Any _
) As Long
Private Declare Function SetBkColor Lib "gdi32" ( _
  ByVal hDC As hDC, _
  ByVal crColor As Long _
) As Long
'----------------------------------------------------

Private Type BLENDFUNCTION
    BlendOp             As Byte
    BlendFlags          As Byte
    SourceConstantAlpha As Byte
    AlphaFormat         As Byte
End Type
'----------------------------------------------------

Private Const CmPerInch             As Double = 2.53968253968254
Private Const TwipsPerInch          As Long = 1440
Private hmX         As Double
Private hmY         As Double
'----------------------------------------------------

Private m_Target    As CDraw
Private m_PictureDC As hDC
Private m_hPicture  As HANDLE
Private m_lWidth    As Long, _
        m_lHeight   As Long
Private m_bInit     As Boolean
'----------------------------------------------------

Private Sub Class_Initialize()
    ' factors for OLE_X/YSIZE_HIMETRIC-to-pixel transformation
    hmX = 1 / 1000 / CmPerInch * TwipsPerInch / Screen.TwipsPerPixelX
    hmY = 1 / 1000 / CmPerInch * TwipsPerInch / Screen.TwipsPerPixelY
End Sub

Private Sub Class_Terminate()
    Call Clear
End Sub
'----------------------------------------------------

Public Property Get Target() As CDraw
    Set Target = m_Target
End Property
Public Property Set Target(Value As CDraw)
    Set m_Target = Value
End Property

Public Property Get Width() As Long
    Width = m_lWidth
End Property

Public Property Get Height() As Long
    Height = m_lHeight
End Property
'----------------------------------------------------

'----------------------------------------------------
' LoadFromFile
' loads a bitmap from a file
' - [IN] FileName: path to bitmap file
'----------------------------------------------------
Public Sub LoadFromFile(FileName As String)
    ' REALLY cool idea from Lucky's VB Gaming Site
    ' (http://www.rookscape.com/vbgaming/tutW.php)
    Dim tmpPic  As IPictureDisp
    
    Call Clear
    
    m_PictureDC = CreateCompatibleDC(0)
    
    Set tmpPic = LoadPicture(FileName)
    m_lWidth = tmpPic.Width * hmX
    m_lHeight = tmpPic.Height * hmY
    m_hPicture = SelectObject(m_PictureDC, tmpPic.HANDLE)
    
    m_bInit = True
End Sub

'----------------------------------------------------
' LoadFromResource
' loads a bitmap from a resource
' - [IN] ID As String: bitmap resource ID
'----------------------------------------------------
Public Sub LoadFromResource(ID As String)
    Dim tmpPic  As IPictureDisp
    
    Call Clear
    
    m_PictureDC = CreateCompatibleDC(0)
    
    Set tmpPic = LoadResPicture(ID, vbResBitmap)
    m_lWidth = tmpPic.Width * hmX
    m_lHeight = tmpPic.Height * hmY
    m_hPicture = SelectObject(m_PictureDC, tmpPic.HANDLE)
    
    m_bInit = True
End Sub
'----------------------------------------------------

'----------------------------------------------------
' DrawSpriteFast
' blits the bitmap
' - [IN] x As Long: position of upper left corner x
' - [IN] y As Long: position of upper left corner y
'----------------------------------------------------
Public Sub DrawSpriteFast( _
  Optional x As Long, _
  Optional y As Long _
)
    If m_bInit Then _
        Call BitBlt( _
            m_Target.DC, _
            x, y, m_lWidth, m_lHeight, _
            m_PictureDC, _
            0, 0, _
            vbSrcCopy _
        )
End Sub

'----------------------------------------------------
' TransparentDraw
' blits the bitmap with a colourkey for transparency
' - [IN] ColourKey As OLE_COLOR: colour of
'   transparent area
' - [IN] x As Long: position of upper left corner x
' - [IN] y As Long: position of upper left corner y
'----------------------------------------------------
Public Sub TransparentDraw( _
  ColourKey As OLE_COLOR, _
  Optional x As Long, _
  Optional y As Long _
)
    If m_bInit Then _
        Call TransparentBlt( _
            m_Target.DC, _
            x, y, m_lWidth, m_lHeight, _
            m_PictureDC, _
            0, 0, m_lWidth, m_lHeight, _
            ColourKey _
        )
End Sub

'----------------------------------------------------
' AlphaDraw
' alphablends the bitmap
' - [IN] Alpha As Double: value between 0 and 1,
'   indicates degree of blending
' - [IN] x As Long: position of upper left corner x
' - [IN] y As Long: position of upper left corner y
'----------------------------------------------------
Public Sub AlphaDraw( _
  Alpha As Double, _
  x As Long, _
  y As Long _
)
    ' Code from VBFrood
    ' (http://perso.wanadoo.fr/vbfrood/deutsch/projects/fade.htm)
    Dim Blender As BLENDFUNCTION
    Dim LngMap  As Long
    
    If m_bInit Then
        Blender.SourceConstantAlpha = Alpha * 255
        
        Call Peek(LngMap, VarPtr(Blender), LenB(Blender))
        
        Call AlphaBlend( _
            m_Target.DC, _
            x, y, m_lWidth, m_lHeight, _
            m_PictureDC, _
            0, 0, m_lWidth, m_lHeight, _
            LngMap _
        )
    End If
End Sub

' function is slow and sometimes erroneous - take other algorithm
#If False Then '----------------------------------------------------
Public Sub TransBlt( _
  ColourKey As OLE_COLOR, _
  Alpha As Double, _
  Optional x As Long, _
  Optional y As Long _
)
    Dim tmpDC   As hDC, _
        ResDC   As hDC
    Dim tmpPic  As HANDLE, _
        prevPic As HANDLE, _
        ResPic  As HANDLE, _
        pResPic As HANDLE
    Dim BkCol   As Long
    
    ' create mem DC
    tmpDC = CreateCompatibleDC(0)
    tmpPic = CreateBitmap( _
        m_lWidth, m_lHeight, 1&, 1&, ByVal 0& _
    )
    prevPic = SelectObject(tmpDC, tmpPic)
    BkCol = SetBkColor(m_PictureDC, ColourKey)
    
    ' create b/w mask
    Call BitBlt( _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        m_PictureDC, _
        0, 0, _
        vbSrcCopy _
    )
    Call SetBkColor(m_PictureDC, BkCol)
    
    ' invert mask
    Call BitBlt( _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        0&, _
        0, 0, _
        vbDstInvert _
    )
    
    ' new DC
    ResDC = CreateCompatibleDC(0)
    ResPic = CreateCompatibleBitmap( _
        m_Target.DC, m_Target.Width, m_Target.Height _
    )
    pResPic = SelectObject(ResDC, ResPic)
    
    Call BitBlt( _
        ResDC, _
        0, 0, m_Target.Width, m_Target.Height, _
        m_Target.DC, _
        x, y, _
        vbSrcCopy _
    )
    
    ' alpha blend
    Dim Blender As BLENDFUNCTION
    Dim LngMap  As Long
    
    Blender.SourceConstantAlpha = Alpha * 255
    
    Call Peek(LngMap, VarPtr(Blender), LenB(Blender))
    
    Call AlphaBlend( _
        ResDC, _
        0, 0, m_lWidth, m_lHeight, _
        m_PictureDC, _
        0, 0, m_lWidth, m_lHeight, _
        LngMap _
    )
    ' / alpha blend
    
    Call BitBlt( _
        ResDC, _
        0, 0, m_lWidth, m_lHeight, _
        tmpDC, _
        0, 0, _
        vbSrcAnd _
    )
    
    Call BitBlt( _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        0&, _
        0, 0, _
        vbDstInvert _
    )
    
    Call BitBlt( _
        m_Target.DC, _
        x, y, m_lWidth, m_lHeight, _
        tmpDC, _
        0, 0, _
        vbSrcAnd _
    )
    
    Call BitBlt( _
        m_Target.DC, _
        x, y, m_lWidth, m_lHeight, _
        ResDC, _
        0, 0, _
        vbSrcPaint _
    )
    
    Call SelectObject(ResDC, pResPic)
    Call GDIDeleteObject(ResPic)
    Call DeleteDC(ResDC)
    Call SelectObject(tmpDC, prevPic)
    Call GDIDeleteObject(tmpPic)
    Call DeleteDC(tmpDC)
End Sub
#End If '----------------------------------------------------

'----------------------------------------------------
' TransparentAlphaDraw As
' alphablends the bitmap while respecting transparency
' - [IN] ColourKey As OLE_COLOR: colour of
'   transparent area
' - [IN] Alpha As Double: value between 0 and 1,
'   indicates degree of blending
' - [IN] x As Long: position of upper left corner x
' - [IN] y As Long: position of upper left corner y
'----------------------------------------------------
Public Sub TransparentAlphaDraw( _
  ColourKey As OLE_COLOR, _
  Alpha As Double, _
  Optional x As Long, _
  Optional y As Long _
)
    Dim tmpDC   As hDC
    Dim tmpPic  As HANDLE, _
        prevPic As HANDLE
    Dim BkCol   As Long
    
    tmpDC = CreateCompatibleDC(0)
    tmpPic = CreateCompatibleBitmap( _
        m_Target.DC, m_Target.Width, m_Target.Height _
    )
    prevPic = SelectObject(tmpDC, tmpPic)
    
    'copy rect from BackBuffer
    Call BitBlt( _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        m_Target.DC, _
        x, y, _
        vbSrcCopy _
    )
    
    'blt transparent sprite
    Call TransparentBlt( _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        m_PictureDC, _
        0, 0, m_lWidth, m_lHeight, _
        ColourKey _
    )
    
    ' alpha blend
    Dim Blender As BLENDFUNCTION
    Dim LngMap  As Long
    
    Blender.SourceConstantAlpha = Alpha * 255
    
    Call Peek(LngMap, VarPtr(Blender), LenB(Blender))
    
    Call AlphaBlend( _
        m_Target.DC, _
        x, y, m_lWidth, m_lHeight, _
        tmpDC, _
        0, 0, m_lWidth, m_lHeight, _
        LngMap _
    )
    ' / alpha blend
    
    Call SelectObject(tmpDC, prevPic)
    Call GDIDeleteObject(tmpPic)
    Call DeleteDC(tmpDC)
End Sub
'----------------------------------------------------

'----------------------------------------------------
' Clear
' clears up the memory
'----------------------------------------------------
Private Sub Clear()
    If m_bInit Then
        Call SelectObject(m_PictureDC, m_hPicture)
        Call GDIDeleteObject(m_hPicture)
        Call DeleteDC(m_PictureDC)
    End If
End Sub
