VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CDraw"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private Const CLASSNAME As String = "CDraw"

Private m_BackBuffer    As CBackBuffer
Private m_hViewPortWnd  As hWnd
Private m_hViewPortDC   As hDC
Private m_lWidth        As Long, _
        m_lHeight       As Long
Private m_bObjLoaded    As Boolean

Private m_Pens          As CObjectsGDI
Private m_Brushes       As CObjectsGDI
Private m_Fonts         As CObjectsGDI

Private m_DrawObjects   As CDrawingObjects

Private m_hOldPen       As HANDLE
Private m_hOldBrush     As HANDLE
Private m_hOldFont      As HANDLE
'----------------------------------------------------

Public Property Get ViewPortWnd() As hWnd
    ViewPortWnd = m_hViewPortWnd
End Property
Public Property Let ViewPortWnd(Value As hWnd)
    m_hViewPortWnd = Value
    
    Call MakeNewBackBuffer
End Property

Public Property Set ViewPort(Value As Object)
    Dim OldScale As ScaleModeConstants
    
    If Not TypeOf Value Is PictureBox And _
        Not TypeOf Value Is Form _
    Then _
        Call Err.Raise( _
            vbObjectError, _
            CLASSNAME & ":ViewPort", _
            "Invalid target control" _
        )
    
    m_hViewPortWnd = Value.hWnd
    
    OldScale = Value.ScaleMode
    Value.ScaleMode = vbPixels
    
    m_lWidth = Value.ScaleWidth
    m_lHeight = Value.ScaleHeight
    
    Value.ScaleMode = OldScale
    
    Call MakeNewBackBuffer
End Property

Public Property Get DC() As hDC
    DC = m_BackBuffer.DC
End Property

Public Property Get Drawing() As CDrawingObjects
    Set Drawing = m_DrawObjects
End Property

Public Property Get Width() As Long
    Width = m_lWidth
End Property

Public Property Get Height() As Long
    Height = m_lHeight
End Property
'----------------------------------------------------

'----------------------------------------------------
' AddPen As HANDLE
' creates a pen
' - [IN] Style As PenStyles: pen style
' - [IN] Width As Long: pen width
' - [IN] Colour As OLE_COLOUR: pen colour
' - returns handle to pen
'----------------------------------------------------
Public Function AddPen( _
  Style As PenStyles, _
  Width As Long, _
  Colour As OLE_COLOR _
) As HANDLE
    AddPen = m_Pens.Add( _
        CreatePen(Style, Width, Colour) _
    )
End Function

'----------------------------------------------------
' RemovePen
' removes a pen
' - [IN] Index As HANDLE: handle to pen
'----------------------------------------------------
Public Sub RemovePen(Index As HANDLE)
    Call m_Pens.Remove(Index)
End Sub

'----------------------------------------------------
' AddSolidBrush As HANDLE
' creates a solid brush
' - [IN] Colour As OLE_COLOUR: brush colour
' - returns handle to brush
'----------------------------------------------------
Public Function AddSolidBrush( _
  Colour As OLE_COLOR _
) As HANDLE
    AddSolidBrush = m_Brushes.Add( _
        CreateSolidBrush(Colour) _
    )
End Function

'----------------------------------------------------
' AddBrush As HANDLE
' creates a brush
' - [IN] Style As BrushStyle: brush style
' - [IN] Colour As OLE_COLOUR: brush colour
' - [IN] Hatch As HatchStyle: brush hatch style
' - returns handle to brush
'----------------------------------------------------
Public Function AddBrush( _
  Style As BrushStyles, _
  Colour As OLE_COLOR, _
  Hatch As HatchStyle _
) As HANDLE
    Dim tmpLogBrush As LOGBRUSH
    
    tmpLogBrush.lbStyle = Style
    tmpLogBrush.lbColor = Colour
    tmpLogBrush.lbHatch = Hatch
    
    AddBrush = m_Brushes.Add( _
        CreateBrushIndirect(tmpLogBrush) _
    )
End Function

'----------------------------------------------------
' RemoveBrush
' removes a brush
' - [IN] Index As HANDLE: handle to brush
'----------------------------------------------------
Public Sub RemoveBrush(Index As HANDLE)
    Call m_Brushes.Remove(Index)
End Sub

'----------------------------------------------------
' AddFont As HANDLE
' creates a font
' - [IN] FontFace As String: font name
' - [IN] FontSize As Long: font size
' - [IN] FontWeight As FontWeight: font weight
' - [IN] Italic As Boolean: font italic
' - [IN] Underline As Boolean: font underline
' - [IN] StrikeTrough As Boolean: font strikethrough
' - [IN] Orientation As Double: angle from baseline
' - returns handle to font
'----------------------------------------------------
Public Function AddFont( _
  FontFace As String, _
  FontSize As Long, _
  FontWeight As FontWeight, _
  Italic As Boolean, _
  Underline As Boolean, _
  StrikeThrough As Boolean, _
  Orientation As Double _
) As HANDLE
    Dim FItalic As BOOL
    Dim FULine  As BOOL
    Dim FStThru As BOOL
    
    Italic = FItalic = [True]
    Underline = FULine = [True]
    StrikeThrough = FStThru = [True]
    
    AddFont = m_Fonts.Add( _
        CreateFont( _
            -MulDiv( _
                FontSize, _
                GetDeviceCaps(m_BackBuffer.DC, LOGPIXELSY), _
                72 _
            ), _
            0&, _
            CLng(10 * Orientation), _
            CLng(10 * Orientation), _
            FontWeight, _
            FItalic, _
            FULine, _
            FStThru, _
            OEM_CHARSET, _
            OUT_DEFAULT_PRECIS, _
            CLIP_DEFAULT_PRECIS, _
            DEFAULT_QUALITY, 0&, _
            FontFace _
        ) _
    )
End Function

'----------------------------------------------------
' RemoveFont
' removes a font
' - [IN] Index as HANDLE: handle to font
'----------------------------------------------------
Public Sub RemoveFont(Index As HANDLE)
    Call m_Fonts.Remove(Index)
End Sub
'----------------------------------------------------

'----------------------------------------------------
' SetPen
' selects a pen
' - [IN] Index As HANDLE: handle to pen
'----------------------------------------------------
Public Sub SetPen(Index As HANDLE)
    Call SelectObject( _
        m_BackBuffer.DC, m_Pens.Item(Index) _
    )
End Sub

'----------------------------------------------------
' SetBrush
' selects a brush
' - [IN] Index As HANDLE: handle to brush
'----------------------------------------------------
Public Sub SetBrush(Index As HANDLE)
    Call SelectObject( _
        m_BackBuffer.DC, m_Brushes.Item(Index) _
    )
End Sub

'----------------------------------------------------
' SetFont
' selects a font
' - [IN] Index As HANDLE: handle to font
'----------------------------------------------------
Public Sub SetFont(Index As HANDLE)
    Call SelectObject( _
        m_BackBuffer.DC, m_Fonts.Item(Index) _
    )
End Sub
'----------------------------------------------------

Public Property Get TextColour() As OLE_COLOR
    TextColour = GetTextColor(m_BackBuffer.DC)
End Property
Public Property Let TextColour(Value As OLE_COLOR)
    Call SetTextColor(m_BackBuffer.DC, Value)
End Property
'----------------------------------------------------

'----------------------------------------------------
' BitBlt
' blits another DC onto this one
' - [IN] Source As hDC: source
' - [IN] SrcLeft As Long: source's left border
' - [IN] SrcTop As Long: source's top border
' - [IN] Width As Long: source's width
' - [IN] Height As Long: source's height
' - [IN] [DestLeft As Long]: destination's left border
' - [IN] [DestTop As Long]: destination's top border
' - [IN] [ROP As RasterOpConstants]: raster operation
'----------------------------------------------------
Public Sub BitBlt( _
  Source As hDC, _
  SrcLeft As Long, _
  SrcTop As Long, _
  Width As Long, _
  Height As Long, _
  Optional DestLeft As Long, _
  Optional DestTop As Long, _
  Optional ROP As RasterOpConstants = vbSrcCopy _
)
    Call modGDI.BitBlt( _
        m_BackBuffer.DC, _
        DestLeft, _
        DestTop, _
        Width, _
        Height, _
        Source, _
        SrcLeft, _
        SrcTop, _
        ROP _
    )
End Sub

'----------------------------------------------------
' TextOut
' draws a text
' - [IN] Text As String: text
' - [IN] x As Long: x position
' - [IN] y As Long: y position
'----------------------------------------------------
Public Sub TextOut( _
  Text As String, _
  x As Long, _
  y As Long _
)
    Call modGDI.TextOut( _
        m_BackBuffer.DC, _
        x, y, _
        Text, _
        LenB(Text) \ 2 _
    )
End Sub

'----------------------------------------------------
' Flip
' flips the backbuffer onto the frontbuffer
'----------------------------------------------------
Public Sub Flip()
    Call m_BackBuffer.BltDC(m_hViewPortDC)
End Sub

'----------------------------------------------------
' Clear
' clears the background
' - [IN] [Colour As OLE_COLOR]: background colour
'----------------------------------------------------
Public Sub Clear(Optional Colour As OLE_COLOR = vbBlack)
    Call modGDI.Clear( _
        m_BackBuffer.DC, _
        Colour, _
        m_lWidth, _
        m_lHeight _
    )
End Sub

'----------------------------------------------------
'----------------------------------------------------
' MakeNewBackBuffer
' creates a new backbuffer
'----------------------------------------------------
Private Sub MakeNewBackBuffer()
    Set m_BackBuffer = _
        NewCBackBuffer( _
            GetDC(m_hViewPortWnd), _
            m_lWidth, _
            m_lHeight _
        )
    
    m_hViewPortDC = GetDC(m_hViewPortWnd)
    m_DrawObjects.TargetDC = m_BackBuffer.DC
    
    m_hOldPen = SelectObject(m_BackBuffer.DC, m_hOldPen)
    m_hOldBrush = SelectObject(m_BackBuffer.DC, m_hOldBrush)
    m_hOldFont = SelectObject(m_BackBuffer.DC, m_hOldFont)
    
    m_bObjLoaded = True
End Sub

Private Sub Class_Initialize()
    Set m_Pens = New CObjectsGDI
    Set m_Brushes = New CObjectsGDI
    Set m_Fonts = New CObjectsGDI
    
    Set m_DrawObjects = New CDrawingObjects
End Sub

Private Sub Class_Terminate()
    Dim I   As Long
    
    If m_bObjLoaded Then
        Call SelectObject(m_BackBuffer.DC, m_hOldPen)
        
        For I = 0 To m_Pens.Count
            If m_Pens.Item(I) <> NULL_HANDLE Then _
                Call GDIDeleteObject(m_Pens.Item(I))
        Next I
        
        Call SelectObject(m_BackBuffer.DC, m_hOldBrush)
        
        For I = 0 To m_Brushes.Count
            If m_Brushes.Item(I) <> NULL_HANDLE Then _
                Call GDIDeleteObject(m_Brushes.Item(I))
        Next I
        
        Call SelectObject(m_BackBuffer.DC, m_hOldFont)
        
        For I = 0 To m_Fonts.Count
            If m_Fonts.Item(I) <> NULL_HANDLE Then _
                Call GDIDeleteObject(m_Fonts.Item(I))
        Next I
        
        Call ReleaseDC(m_hViewPortWnd, m_hViewPortDC)
        
        Set m_BackBuffer = Nothing
        Set m_Pens = Nothing
        Set m_Brushes = Nothing
        Set m_DrawObjects = Nothing
        
        Call DeleteGDIObjects
        
        m_bObjLoaded = False
    End If
End Sub
