VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CDrawingObjects"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************
'* CDrawingObjects.cls                              *
'* handles the drawing via (mapped) GDI functions   *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 16:10                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Private m_hTargetDC As hDC
'----------------------------------------------------

Friend Property Let TargetDC(Value As hDC)
    m_hTargetDC = Value
End Property
'----------------------------------------------------

'----------------------------------------------------
' Locate
' sets the cursor's position
' - [IN] x As Long: x position
' - [IN] y As Long: y position
'----------------------------------------------------
Public Sub Locate(x As Long, y As Long)
    Call modDrawingObjects.MoveToEx(m_hTargetDC, x, y, 0&)
End Sub

'----------------------------------------------------
' DrawLine
' draws a line from the current position
' - [IN] x As Long: x position
' - [IN] y As Long: y position
'----------------------------------------------------
Public Sub DrawLine(x As Long, y As Long)
    Call modDrawingObjects.LineTo(m_hTargetDC, x, y)
End Sub

'----------------------------------------------------
' Point
' draws a point on the given position
' - [IN] x As Long: x position
' - [IN] y As Long: y position
'----------------------------------------------------
Public Sub Point(x As Long, y As Long)
    Call modDrawingObjects.MoveToEx(m_hTargetDC, x, y, 0&)
    Call modDrawingObjects.LineTo(m_hTargetDC, x, y)
End Sub

'----------------------------------------------------
' Arc
' draws an arc
' for parametres see MSDN!
'----------------------------------------------------
Public Sub Arc( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long, _
  X3 As Long, _
  Y3 As Long, _
  X4 As Long, _
  Y4 As Long _
)
    Call modDrawingObjects.Arc( _
        m_hTargetDC, _
        X1, Y1, X2, Y2, X3, Y3, X4, Y4 _
    )
End Sub

'----------------------------------------------------
' Chord
' draws a chord
' for parametres see MSDN!
'----------------------------------------------------
Public Sub Chord( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long, _
  X3 As Long, _
  Y3 As Long, _
  X4 As Long, _
  Y4 As Long _
)
    Call modDrawingObjects.Chord( _
        m_hTargetDC, _
        X1, Y1, X2, Y2, _
        X3, Y3, X4, Y4 _
    )
End Sub

'----------------------------------------------------
' Pie
' draws a pie
' for parametres see MSDN!
'----------------------------------------------------
Public Sub Pie( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long, _
  X3 As Long, _
  Y3 As Long, _
  X4 As Long, _
  Y4 As Long _
)
    Call modDrawingObjects.Pie( _
        m_hTargetDC, _
        X1, Y1, X2, Y2, _
        X3, Y3, X4, Y4 _
    )
End Sub

'----------------------------------------------------
' Ellipse
' draws an ellipse
' for parametres see MSDN!
'----------------------------------------------------
Public Sub Ellipse( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long _
)
    Call modDrawingObjects.Ellipse( _
        m_hTargetDC, _
        X1, Y1, X2, Y2 _
    )
End Sub

'----------------------------------------------------
' Polygon
' draws a polygone (= closed polyline)
' - [IN] Points() As POINTAPI: array of points
'----------------------------------------------------
Public Sub Polygon(Points() As POINTAPI)
    Call modDrawingObjects.Polygon( _
        m_hTargetDC, _
        VarPtr(Points(0)), UBound(Points) + 1 _
    )
End Sub

'----------------------------------------------------
' Polyline
' draws a polyline (= open polygone)
' - [IN] Points() As POINTAPI: array of points
'----------------------------------------------------
Public Sub Polyline(Points() As POINTAPI)
    Call modDrawingObjects.Polyline( _
        m_hTargetDC, _
        VarPtr(Points(0)), UBound(Points) + 1 _
    )
End Sub

'----------------------------------------------------
' Rectangle
' draws a rectangle
' for parametres see MSDN!
'----------------------------------------------------
Public Sub Rectangle( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long _
)
    Call modDrawingObjects.Rectangle( _
        m_hTargetDC, _
        X1, Y1, X2, Y2 _
    )
End Sub

'----------------------------------------------------
' RoundRect
' draws a rectangle with rounded corners
' for parametres see MSDN!
'----------------------------------------------------
Public Sub RoundRect( _
  X1 As Long, _
  Y1 As Long, _
  X2 As Long, _
  Y2 As Long, _
  X3 As Long, _
  Y3 As Long _
)
    Call modDrawingObjects.RoundRect( _
        m_hTargetDC, _
        X1, Y1, X2, Y2, X3, Y3 _
    )
End Sub

'----------------------------------------------------
' Star
' draws a star
' - [IN] x As Long: x position of centre
' - [IN] y As Long: y position of centre
' - [IN] InnerRadius As Long: inner radius
' - [IN] OuterRadius As Long: outer radius
' - [IN] Count As Long: number of spikes
' - [IN] Phase As Double: shift of first spike from
'   12h position (in rad)
'----------------------------------------------------
Public Sub Star( _
  x As Long, _
  y As Long, _
  InnerRadius As Long, _
  OuterRadius As Long, _
  Count As Long, _
  Phase As Double _
)
    Dim Strip() As POINTAPI
    Dim I       As Long
    Dim Dist    As Double
    Const Pi    As Double = 3.1415926535
    
    ReDim Strip(Count * 2 - 1) As POINTAPI
    
    Dist = Pi / Count
    
    For I = 0 To Count - 1
        Strip(I * 2).x = _
                        Cos(I * 2 * Dist - Phase) * _
                        OuterRadius + x
        Strip(I * 2).y = _
                        Sin(I * 2 * Dist - Phase) * _
                        OuterRadius + y
        
        Strip(I * 2 + 1).x = _
                        Cos((I * 2 + 1) * Dist - Phase) * _
                        InnerRadius + x
        Strip(I * 2 + 1).y = _
                        Sin((I * 2 + 1) * Dist - Phase) * _
                        InnerRadius + y
    Next I
    
    Call modDrawingObjects.Polygon( _
        m_hTargetDC, _
        VarPtr(Strip(0)), Count * 2 _
    )
End Sub
