VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CGradient"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************
'* CGradient.cls                                    *
'* draws fast gradients via DIB                     *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 16:18                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Private Type BITMAPINFOHEADER
    biSize          As Long
    biWidth         As Long
    biHeight        As Long
    biPlanes        As Integer
    biBitCount      As Integer
    biCompression   As Long
    biSizeImage     As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed       As Long
    biClrImportant  As Long
End Type

Private Type RGBQUAD
    rgbBlue         As Byte
    rgbGreen        As Byte
    rgbRed          As Byte
    rgbReserved     As Byte
End Type

Private Type BITMAPINFO
    bmiHeader       As BITMAPINFOHEADER
    bmiColors(255)  As RGBQUAD
End Type

Public Enum GradientDirection
    GD_HORIZONTALLY = 0
    GD_VERICALLY = 1
End Enum

Private Declare Function StretchDIBits Lib "gdi32" ( _
  ByVal hDC As hDC, _
  ByVal x As Long, _
  ByVal y As Long, _
  ByVal dx As Long, _
  ByVal dy As Long, _
  ByVal SrcX As Long, _
  ByVal SrcY As Long, _
  ByVal wSrcWidth As Long, _
  ByVal wSrcHeight As Long, _
  lpBits As Any, _
  lpBitsInfo As BITMAPINFO, _
  ByVal wUsage As Long, _
  ByVal dwRop As Long _
) As Long
'----------------------------------------------------

Private m_Target    As CDraw
'----------------------------------------------------

Public Property Get Target() As CDraw
    Set Target = m_Target
End Property
Public Property Set Target(Value As CDraw)
    Set m_Target = Value
End Property
'----------------------------------------------------

'----------------------------------------------------
' DrawGradient
' draws a gradient on a target
' - [IN] Target As Long: hDC of target
' - [IN] Position As RECT: RECT structure indicating where to paint
' - [IN] colFrom As OLE_COLOR: starting colour
' - [IN] colTo As OLE_COLOR: ending colour
' - [IN] Direction As GradientDirection: horizontally or vertically
'----------------------------------------------------
Public Sub DrawGradient( _
  Position As RECT, _
  colFrom As OLE_COLOR, _
  colTo As OLE_COLOR, _
  Optional Direction As GradientDirection = GD_HORIZONTALLY _
)
    Dim Target      As hDC
    Dim Gradient()  As Byte
    Dim Info        As BITMAPINFO
    Dim I           As Long
    Dim Width       As Long, _
        Height      As Long
    Dim StepR       As Single, _
        StepG       As Single, _
        StepB       As Single
    Dim StartR      As Byte, _
        StartG      As Byte, _
        StartB      As Byte
    Dim EndR        As Byte, _
        EndG        As Byte, _
        EndB        As Byte
    Dim CurrentR    As Single, _
        CurrentG    As Single, _
        CurrentB    As Single
    
    Target = m_Target.DC
    
    With Position
        Width = .Right - .Left
        Height = .Bottom - .Top
    End With
    
    ' set up bmp header
    With Info.bmiHeader
        .biBitCount = 24
        .biSize = Len(Info.bmiHeader)
        .biPlanes = 1
    End With
    
    ' get start colors
    StartR = colFrom And vbRed
    StartG = (colFrom And vbGreen) \ &H100
    StartB = (colFrom And vbBlue) \ &H10000
    
    ' get end colors
    EndR = colTo And vbRed
    EndG = (colTo And vbGreen) \ &H100
    EndB = (colTo And vbBlue) \ &H10000
    
    CurrentR = StartR
    CurrentG = StartG
    CurrentB = StartB
    
    Select Case Direction
    Case GradientDirection.GD_HORIZONTALLY
        ReDim Gradient(3 * Width - 1) As Byte
        
        ' set up bmp dimensions
        With Info.bmiHeader
            .biWidth = Width
            .biHeight = 1
            .biSizeImage = Width
        End With
        
        ' get step size
        StepR = (CSng(EndR) - CSng(StartR)) / Width
        StepG = (CSng(EndG) - CSng(StartG)) / Width
        StepB = (CSng(EndB) - CSng(StartB)) / Width
        
        ' loop through bitmap and set colors
        For I = 0 To Width - 1
            Gradient(I * 3) = CurrentB
            Gradient(I * 3 + 1) = CurrentG
            Gradient(I * 3 + 2) = CurrentR
            
            CurrentR = CurrentR + StepR
            CurrentG = CurrentG + StepG
            CurrentB = CurrentB + StepB
        Next
        
        ' blit bmp
        Call StretchDIBits( _
            Target, _
            Position.Left, Position.Top, _
            Width, Height, _
            0, 0, Width, 1, Gradient(0), _
            Info, 0, vbSrcCopy _
        )
    
    Case GradientDirection.GD_VERICALLY
        ReDim Gradient(4 * 3 * Height - 1) As Byte
        
        ' set up bmp dimensions
        With Info.bmiHeader
            .biWidth = 4
            .biHeight = Height
            .biSizeImage = Height
        End With
        
        ' get step size
        StepR = (CSng(EndR) - CSng(StartR)) / Height / 4
        StepG = (CSng(EndG) - CSng(StartG)) / Height / 4
        StepB = (CSng(EndB) - CSng(StartB)) / Height / 4
        
        ' loop through bitmap and set colors
        For I = 0 To 4 * Height - 1
            Gradient(I * 3) = CurrentB
            Gradient(I * 3 + 1) = CurrentG
            Gradient(I * 3 + 2) = CurrentR
            
            CurrentR = CurrentR + StepR
            CurrentG = CurrentG + StepG
            CurrentB = CurrentB + StepB
        Next
        
        ' blit bmp
        Call StretchDIBits( _
            Target, _
            Position.Left, Position.Top, _
            Width, Height, _
            0, 0, 4, Height, Gradient(0), _
            Info, 0, vbSrcCopy _
        )
    End Select
End Sub

'----------------------------------------------------
' DrawGradientEx
' draws a multicolor gradient on a target
' - [IN] Target As Long: hDC of target
' - [IN] Position As RECT: RECT structure indicating where to paint
' - [IN] Colors() As OLE_COLOR: array of colours to use
' - [IN] Direction As GradientDirection: horizontally or vertically
'----------------------------------------------------
Public Sub DrawGradientEx( _
  Position As RECT, _
  Colors() As OLE_COLOR, _
  Optional Direction As GradientDirection = GD_HORIZONTALLY _
)
    Dim Bounds  As Long
    Dim I       As Long
    Dim Diff    As Single
    Dim ActRect As RECT
    Dim Target  As hDC
       
    Target = m_Target.DC
    
    Bounds = UBound(Colors)
    If LBound(Colors) Or Bounds < 1 Then _
        Exit Sub
    
    Select Case Direction
    Case GradientDirection.GD_HORIZONTALLY
        Diff = (Position.Right - Position.Left) / Bounds
        
        With ActRect
            .Left = Position.Left
            .Top = Position.Top
            .Right = .Left + Diff
            .Bottom = Position.Bottom
        End With
        
        For I = 1 To Bounds
            Call DrawGradient( _
                ActRect, _
                Colors(I - 1), Colors(I), Direction _
            )
            With ActRect
                .Left = .Left + Diff
                .Right = .Right + Diff
            End With
        Next
    Case GradientDirection.GD_VERICALLY
        Diff = (Position.Bottom - Position.Top) / Bounds
        
        ' goes from bottom to top!
        With ActRect
            .Left = Position.Left
            .Bottom = Position.Bottom
            .Right = Position.Right
            .Top = .Bottom - Diff
        End With
        
        For I = 1 To Bounds
            Call DrawGradient( _
                ActRect, _
                Colors(I - 1), Colors(I), Direction _
            )
            With ActRect
                .Top = .Top - Diff
                .Bottom = .Bottom - Diff
            End With
        Next
    End Select
End Sub
