VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CSpace"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'****************************************************
'* CSpace.cls                                       *
'* 3D space class                                   *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 16:22                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Private Type RenderStructure
    TriangleStrip() As POINT3D
    Flattened()     As POINTAPI
    Enabled         As Boolean
    Visible         As Boolean
End Type
'----------------------------------------------------

Private m_Target    As CDraw
Private m_hPen      As HANDLE
Private m_Colour    As OLE_COLOR
Private m_dZoom     As Double
Private m_dAngle    As Double
Private m_ViewPort  As RECT
Private m_lWidth    As Long, _
        m_lHeight   As Long

Private m_Objects() As RenderStructure
'----------------------------------------------------

Private Sub Class_Initialize()
    Call Clear
End Sub

Private Sub Class_Terminate()
    Call Clear
    
    If m_hPen <> NULL_HANDLE Then _
        Call m_Target.RemovePen(m_hPen)
End Sub
'----------------------------------------------------

Public Property Get Angle() As Double
    Angle = m_dAngle
End Property
Public Property Let Angle(RHS As Double)
    m_dAngle = RHS
    
    Call FlattenAll
End Property

Public Property Get Colour() As OLE_COLOR
    Colour = m_Colour
End Property
Public Property Let Colour(RHS As OLE_COLOR)
    If m_hPen <> NULL_HANDLE Then _
        Call m_Target.RemovePen(m_hPen)
    
    m_Colour = RHS
    m_hPen = m_Target.AddPen( _
        PS_SOLID, 1, m_Colour _
    )
End Property

Public Property Get ViewPort() As RECT
    ViewPort = m_ViewPort
End Property
Public Property Let ViewPort(RHS As RECT)
    m_ViewPort = RHS
    
    Call FlattenAll
End Property
'----------------------------------------------------

'----------------------------------------------------
' Clear
' clears objects
'----------------------------------------------------
Public Sub Clear()
    Erase m_Objects
    ReDim m_Objects(10) As RenderStructure
End Sub

'----------------------------------------------------
' Create3DSpace
' creates a 3D space
' - [IN] Target As CDraw: target
' - [IN] [WireColour As OLE_COLOUR]: colour of wires
'----------------------------------------------------
Public Sub Create3DSpace( _
  Target As CDraw, _
  Optional WireColour As OLE_COLOR = vbWhite _
)
    Set m_Target = Target
    
    Call modTransform.Initialise
    
    m_Colour = WireColour
    
    m_hPen = m_Target.AddPen( _
        PS_SOLID, 1, m_Colour _
    )
    
    ' assign default values
    m_dZoom = 3
    
    With m_ViewPort
        .Left = -10
        .Top = 10
        .Right = 10
        .Bottom = -10
    End With ' m_ViewPort
    
    m_dAngle = 2.5
    
    m_lWidth = m_Target.Width
    m_lHeight = m_Target.Height
End Sub

'----------------------------------------------------
' Add As HANDLE
' adds a trianglestrip
' - [IN] TriangleStrip() As POINT3D: trianglestrip
' - returns object's handle
'----------------------------------------------------
Public Function Add( _
  TriangleStrip() As POINT3D _
) As HANDLE
    Add = GetFreeSlot()
    
    m_Objects(Add).Enabled = True
    m_Objects(Add).Visible = True
    
    m_Objects(Add).TriangleStrip = TriangleStrip
    
    Call Flatten(m_Objects(Add))
End Function

'----------------------------------------------------
' Remove
' removes an object
' - [IN] Index As HANDLE: handle to object
'----------------------------------------------------
Public Sub Remove(Index As HANDLE)
    If Index < UBound(m_Objects) Then
        m_Objects(Index).Enabled = False
        
        Erase m_Objects(Index).TriangleStrip
        Erase m_Objects(Index).Flattened
    End If ' Index < UBound(m_Objects)
End Sub

'----------------------------------------------------
' Render
' paints all objects
'----------------------------------------------------
Public Sub Render()
    Dim I       As Long
    
    Call m_Target.SetPen(m_hPen)
    
    For I = 0 To UBound(m_Objects)
        If m_Objects(I).Enabled And m_Objects(I).Visible Then _
            Call m_Target.Drawing.Polyline( _
                m_Objects(I).Flattened _
            )
    Next I
End Sub
'----------------------------------------------------

'----------------------------------------------------
' Translate
' maps the Translate() function
'----------------------------------------------------
Public Sub Translate( _
  Index As HANDLE, _
  Vector As POINT3D _
)
    Call modTransform.Translate( _
        m_Objects(Index).TriangleStrip, _
        Vector _
    )
    Call Flatten(m_Objects(Index))
End Sub

'----------------------------------------------------
' Stretch
' maps the Stretch() function
'----------------------------------------------------
Public Sub Stretch( _
  Index As HANDLE, _
  Factor As Double _
)
    Call modTransform.Stretch( _
        m_Objects(Index).TriangleStrip, _
        Factor _
    )
    Call Flatten(m_Objects(Index))
End Sub

'----------------------------------------------------
' Rotate
' maps the Rotate() function
'----------------------------------------------------
Public Sub Rotate( _
  Index As HANDLE, _
  Alpha As Long, _
  Beta As Long, _
  Gamma As Long _
)
    Call modTransform.Rotate( _
        m_Objects(Index).TriangleStrip, _
        Alpha, _
        Beta, _
        Gamma _
    )
    Call Flatten(m_Objects(Index))
End Sub
'----------------------------------------------------

'----------------------------------------------------
' GetFreeSlot As HANDLE
' - returns the next free slot
'----------------------------------------------------
Private Function GetFreeSlot() As HANDLE
    For GetFreeSlot = 0 To UBound(m_Objects)
        If Not m_Objects(GetFreeSlot).Enabled Then _
            Exit Function
    Next GetFreeSlot
    
    ' no free slot - enlarge stack
    ReDim Preserve m_Objects(UBound(m_Objects) + 10) As RenderStructure
End Function

'----------------------------------------------------
' FlattenAll
' flattens all objects
'----------------------------------------------------
Private Sub FlattenAll()
    Dim I   As Long
    
    For I = 0 To UBound(m_Objects)
        If m_Objects(I).Enabled Then _
            Call Flatten(m_Objects(I))
    Next I
End Sub

'----------------------------------------------------
' Flatten
' flattens an object (i.e. calculate 2D coordinates)
' - [IN] RenderObject As RenderStructure: object
'----------------------------------------------------
Private Sub Flatten(RenderObject As RenderStructure)
    Dim I       As Long
    Dim Width   As Double, _
        Height  As Double
    Dim DirX    As Double, _
        DirY    As Double
    Dim UB      As Long
    
    UB = UBound(RenderObject.TriangleStrip)
    
    ReDim RenderObject.Flattened(UB) As POINTAPI
    
    With m_ViewPort
        Width = m_lWidth / (.Right - .Left)
        Height = m_lHeight / (.Bottom - .Top)
        DirX = IIf(.Left < .Right, 1, -1) * m_dAngle
        DirY = IIf(.Top > .Bottom, 1, -1) * m_dAngle
    End With ' m_ViewPort
    
    For I = 0 To UB
        RenderObject.Flattened(I).x = (RenderObject.TriangleStrip(I).x + _
                                      RenderObject.TriangleStrip(I).z / DirX) * _
                                      Width - m_ViewPort.Left * Width
        RenderObject.Flattened(I).y = (RenderObject.TriangleStrip(I).y + _
                                      RenderObject.TriangleStrip(I).z / DirY) * _
                                      Height - m_ViewPort.Top * Height
    Next I
End Sub
