Attribute VB_Name = "modTransform"
'****************************************************
'* modTransform.bas                                 *
'* geometrically transforms 3D triangle strips      *
'* Programmed: Konrad Rudolph (Mad Rat Pro.)        *
'* Last Change: 25.08.2003 15:28                    *
'* Version: 1.0.0                                   *
'****************************************************
Option Explicit

Private CosT()  As Double, _
        SinT()  As Double

Private m_bInit As Boolean

'----------------------------------------------------
' Initialise
' inits the sinus and cosinus tables
'----------------------------------------------------
Public Sub Initialise()
    Const Pi As Double = 3.14159265358979
    Const Rad = Pi / 180
    
    Dim I   As Long
    
    If m_bInit Then _
        Exit Sub
    
    ReDim CosT(359) As Double
    ReDim SinT(359) As Double
    
    For I = 0 To 359
        CosT(I) = Cos(I * Rad)
        SinT(I) = Sin(I * Rad)
    Next I
    
    m_bInit = True
End Sub

'----------------------------------------------------
' Translate
' translates an object
' - [IN/OUT] Strip() AS POINT3D: object
' - [IN] Vector As POINT3D: translation vector
'----------------------------------------------------
Public Sub Translate( _
  Strip() As POINT3D, _
  Vector As POINT3D _
)
    Dim I   As Long
    
    For I = 0 To UBound(Strip)
        Strip(I).x = Strip(I).x + Vector.x
        Strip(I).y = Strip(I).y + Vector.y
        Strip(I).z = Strip(I).z + Vector.z
    Next I
End Sub

'----------------------------------------------------
' Stretch
' stretches/shrinks an object
' - [IN/OUT] Strip() AS POINT3D: object
' - [IN] Factor As Double: stretching factor
'----------------------------------------------------
Public Sub Stretch( _
  Strip() As POINT3D, _
  Factor As Double _
)
    Dim I   As Long
    
    For I = 0 To UBound(Strip)
        Strip(I).x = Strip(I).x * Factor
        Strip(I).y = Strip(I).y * Factor
        Strip(I).z = Strip(I).z * Factor
    Next I
End Sub

'----------------------------------------------------
' Rotate
' rotates an object
' - [IN/OUT] Strip() AS POINT3D: object
' - [IN] Alpha As Long: rotation around x axis (in )
' - [IN] Beta As Long: rotation around y axis (in )
' - [IN] Gamma As Long: rotation around z axis (in )
'----------------------------------------------------
Public Sub Rotate( _
  Strip() As POINT3D, _
  Alpha As Long, _
  Beta As Long, _
  Gamma As Long _
)
    Dim I   As Long
    Dim ax  As Double, _
        ay  As Double, _
        az  As Double
    
    Alpha = Alpha Mod 360
    Beta = Beta Mod 360
    Gamma = Gamma Mod 360
    
    ' spin around x axis
    If Alpha <> 0 Then
        For I = 0 To UBound(Strip)
            ay = Strip(I).y
            az = Strip(I).z
            
            Strip(I).y = ay * CosT(Alpha) - az * SinT(Alpha)
            Strip(I).z = ay * SinT(Alpha) + az * CosT(Alpha)
        Next I
    End If
    
    ' spin around y axis
    If Beta <> 0 Then
        For I = 0 To UBound(Strip)
            ax = Strip(I).x
            az = Strip(I).z
            
            Strip(I).x = az * SinT(Beta) + ax * CosT(Beta)
            Strip(I).z = az * CosT(Beta) - ax * SinT(Beta)
        Next I
    End If
    
    ' spin around z axis
    If Gamma <> 0 Then
        For I = 0 To UBound(Strip)
            ax = Strip(I).x
            ay = Strip(I).y
            
            Strip(I).x = ax * CosT(Gamma) - ay * SinT(Gamma)
            Strip(I).y = ax * SinT(Gamma) + ay * CosT(Gamma)
        Next I
    End If
End Sub
